@@ -26,3 +26,13 @@ class InvalidUserId(Exception):
 class CorruptedSaveData(Exception):
     """Raised when the save data can't be read"""
     pass
+
+
+class BadResponse(Exception):
+    """Raised when Instagram returns a non-ok status code."""
+    pass
+
+
+class MissingValue(Exception):
+    """Raised when an action struct is initiated with a missing value"""
+    pass
